const char bsdsocket_lua[] =
"-- bsdsocket.lua (internal file)\n"
"\n"
"do\n"
"\n"
"local TIMEOUT_INFINITY      = 500 * 365 * 86400\n"
"\n"
"local ffi = require 'ffi'\n"
"\n"
"ffi.cdef[[\n"
"    typedef uint32_t socklen_t;\n"
"    typedef ptrdiff_t ssize_t;\n"
"\n"
"    int connect(int sockfd, const struct sockaddr *addr,\n"
"                socklen_t addrlen);\n"
"    int bind(int sockfd, const struct sockaddr *addr,\n"
"                socklen_t addrlen);\n"
"\n"
"    ssize_t write(int fh, const char *octets, size_t len);\n"
"    ssize_t read(int fd, void *buf, size_t count);\n"
"    int listen(int fh, int backlog);\n"
"    int socket(int domain, int type, int protocol);\n"
"    int close(int s);\n"
"    int shutdown(int s, int how);\n"
"    ssize_t send(int sockfd, const void *buf, size_t len, int flags);\n"
"    ssize_t recv(int s, void *buf, size_t len, int flags);\n"
"    int accept(int s, void *addr, void *addrlen);\n"
"    ssize_t sendto(int sockfd, const void *buf, size_t len, int flags,\n"
"                   const struct sockaddr *dest_addr, socklen_t addrlen);\n"
"\n"
"    int\n"
"    bsdsocket_local_resolve(const char *host, const char *port,\n"
"                            struct sockaddr *addr, socklen_t *socklen);\n"
"    int bsdsocket_nonblock(int fh, int mode);\n"
"\n"
"    int setsockopt(int s, int level, int iname, const void *opt, size_t optlen);\n"
"    int getsockopt(int s, int level, int iname, void *ptr, size_t *optlen);\n"
"\n"
"    typedef struct { int active; int timeout; } linger_t;\n"
"\n"
"    struct protoent {\n"
"        char  *p_name;       /* official protocol name */\n"
"        char **p_aliases;    /* alias list */\n"
"        int    p_proto;      /* protocol number */\n"
"    };\n"
"    struct protoent *getprotobyname(const char *name);\n"
"]]\n"
"\n"
"ffi.cdef([[\n"
"struct sockaddr {\n"
"    char _data[256]; /* enough to fit any address */\n"
"};\n"
"]]);\n"
"\n"
"local function sprintf(fmt, ...)\n"
"    return string.format(fmt, ...)\n"
"end\n"
"\n"
"local function printf(fmt, ...)\n"
"    print(sprintf(fmt, ...))\n"
"end\n"
"\n"
"local function get_ivalue(table, key)\n"
"    if type(key) == 'number' then\n"
"        return key\n"
"    end\n"
"    return table[key]\n"
"end\n"
"\n"
"local function get_iflags(table, flags)\n"
"    if flags == nil then\n"
"        return 0\n"
"    end\n"
"    local res = 0\n"
"    if type(flags) ~= 'table' then\n"
"        flags = { flags }\n"
"    end\n"
"    for i, f in pairs(flags) do\n"
"        if table[f] == nil then\n"
"            return nil\n"
"        end\n"
"        res = bit.bor(res, table[f])\n"
"    end\n"
"    return res\n"
"end\n"
"\n"
"local socket_methods  = {}\n"
"socket_methods.errno = function(self)\n"
"    if self['_errno'] == nil then\n"
"        return 0\n"
"    else\n"
"        return self['_errno']\n"
"    end\n"
"end\n"
"\n"
"socket_methods.error = function(self)\n"
"    if self['_errno'] == nil then\n"
"        return nil\n"
"    else\n"
"        return box.errno.strerror(self._errno)\n"
"    end\n"
"end\n"
"\n"
"local addr = ffi.new('struct sockaddr')\n"
"local addr_len = ffi.new('socklen_t[1]')\n"
"socket_methods.sysconnect = function(self, host, port)\n"
"    self._errno = nil\n"
"\n"
"    host = tostring(host)\n"
"    port = tostring(port)\n"
"\n"
"    addr_len[0] = ffi.sizeof(addr)\n"
"    local res = ffi.C.bsdsocket_local_resolve(host, port, addr, addr_len)\n"
"    if res == 0 then\n"
"        res = ffi.C.connect(self.fh, addr, addr_len[0]);\n"
"    end\n"
"    if res == 0 then\n"
"        return true\n"
"    end\n"
"\n"
"    local errno = box.errno()\n"
"    if errno == box.errno.EINPROGRESS then\n"
"        return true\n"
"    end\n"
"\n"
"    self._errno = errno\n"
"    return false\n"
"end\n"
"\n"
"socket_methods.syswrite = function(self, octets)\n"
"    self._errno = nil\n"
"    local done = ffi.C.write(self.fh, octets, string.len(octets))\n"
"    if done < 0 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"    return done\n"
"end\n"
"\n"
"socket_methods.sysread = function(self, len)\n"
"    self._errno = nil\n"
"    local buf = ffi.new('char[?]', len)\n"
"    local res = ffi.C.read(self.fh, buf, len)\n"
"\n"
"    if res < 0 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"\n"
"    buf = ffi.string(buf, res)\n"
"    return buf\n"
"end\n"
"\n"
"socket_methods.nonblock = function(self, nb)\n"
"    self._errno = nil\n"
"\n"
"    local res\n"
"\n"
"    if nb == nil then\n"
"        res = ffi.C.bsdsocket_nonblock(self.fh, 0x80)\n"
"    elseif nb then\n"
"        res = ffi.C.bsdsocket_nonblock(self.fh, 1)\n"
"    else\n"
"        res = ffi.C.bsdsocket_nonblock(self.fh, 0)\n"
"    end\n"
"\n"
"    if res < 0 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"\n"
"    if res == 1 then\n"
"        return true\n"
"    else\n"
"        return false\n"
"    end\n"
"end\n"
"\n"
"local function wait_safely(self, what, timeout)\n"
"    local f = box.fiber.self()\n"
"    local fid = f:id()\n"
"\n"
"    if self.waiters == nil then\n"
"        self.waiters = {}\n"
"    end\n"
"\n"
"    self.waiters[fid] = f\n"
"    local res = box.socket.internal.iowait(self.fh, what, timeout)\n"
"    self.waiters[fid] = nil\n"
"    return res\n"
"end\n"
"\n"
"socket_methods.readable = function(self, timeout)\n"
"    self._errno = nil\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    local wres = wait_safely(self, 0, timeout)\n"
"\n"
"    if wres == 0 then\n"
"        self._errno = box.errno.ETIMEDOUT\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.wait = function(self, timeout)\n"
"    self._errno = nil\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    local wres = wait_safely(self, 2, timeout)\n"
"\n"
"    if wres == 0 then\n"
"        self._errno = box.errno.ETIMEDOUT\n"
"        return\n"
"    end\n"
"\n"
"    local res = ''\n"
"    if bit.band(wres, 1) ~= 0 then\n"
"        res = res .. 'R'\n"
"    end\n"
"    if bit.band(wres, 2) ~= 0 then\n"
"        res = res .. 'W'\n"
"    end\n"
"    return res\n"
"end\n"
"\n"
"socket_methods.writable = function(self, timeout)\n"
"    self._errno = nil\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    local wres = wait_safely(self, 1, timeout)\n"
"\n"
"    if wres == 0 then\n"
"        self._errno = box.errno.ETIMEDOUT\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.listen = function(self, backlog)\n"
"    self._errno = nil\n"
"    if backlog == nil then\n"
"        backlog = 256\n"
"    end\n"
"    local res = ffi.C.listen(self.fh, backlog)\n"
"    if res < 0 then\n"
"        self._errno = box.errno()\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.bind = function(self, host, port)\n"
"    self._errno = nil\n"
"\n"
"    host = tostring(host)\n"
"    port = tostring(port)\n"
"\n"
"    addr_len[0] = ffi.sizeof(addr)\n"
"    local res = ffi.C.bsdsocket_local_resolve(host, port, addr, addr_len)\n"
"    if res == 0 then\n"
"        res = ffi.C.bind(self.fh, addr, addr_len[0]);\n"
"    end\n"
"    if res == 0 then\n"
"        return true\n"
"    end\n"
"\n"
"    self._errno = box.errno()\n"
"    return false\n"
"end\n"
"\n"
"socket_methods.close = function(self)\n"
"    if self.waiters ~= nil then\n"
"        for fid, fiber in pairs(self.waiters) do\n"
"            fiber:wakeup()\n"
"            self.waiters[fid] = nil\n"
"        end\n"
"    end\n"
"\n"
"    self._errno = nil\n"
"    if ffi.C.close(self.fh) < 0 then\n"
"        self._errno = box.errno()\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.shutdown = function(self, how)\n"
"    local hvariants = {\n"
"        ['R']           = 0,\n"
"        ['READ']        = 0,\n"
"        ['W']           = 1,\n"
"        ['WRITE']       = 1,\n"
"        ['RW']          = 2,\n"
"        ['READ_WRITE']  = 2,\n"
"\n"
"        [0]             = 0,\n"
"        [1]             = 1,\n"
"        [2]             = 2\n"
"    }\n"
"    local ihow = hvariants[how]\n"
"\n"
"    if ihow == nil then\n"
"        ihow = 3\n"
"    end\n"
"    self._errno = nil\n"
"    if ffi.C.shutdown(self.fh, ihow) < 0 then\n"
"        self._errno = box.errno()\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.setsockopt = function(self, level, name, value)\n"
"    local info = get_ivalue(box.socket.internal.SO_OPT, name)\n"
"\n"
"    if info == nil then\n"
"        error(sprintf(\"Unknown socket option name: %s\", tostring(name)))\n"
"    end\n"
"\n"
"    if not info.rw then\n"
"        error(sprintf(\"Socket option %s is read only\", name))\n"
"    end\n"
"\n"
"    self._errno = nil\n"
"\n"
"    if type(level) == 'string' then\n"
"        if level == 'SOL_SOCKET' then\n"
"            level = box.socket.internal.SOL_SOCKET\n"
"        else\n"
"            local p = ffi.C.getprotobyname(level)\n"
"            if p == nil then\n"
"                self._errno = box.errno()\n"
"                return false\n"
"            end\n"
"            level = p.p_proto\n"
"        end\n"
"    else\n"
"        level = tonumber(level)\n"
"    end\n"
"\n"
"    if type(value) == 'boolean' then\n"
"        if value then\n"
"            value = 1\n"
"        else\n"
"            value = 0\n"
"        end\n"
"    end\n"
"\n"
"    if info.type == 1 then\n"
"        local value = ffi.new(\"int[1]\", value)\n"
"        local res = ffi.C.setsockopt(self.fh,\n"
"            level, info.iname, value, ffi.sizeof('int'))\n"
"\n"
"        if res < 0 then\n"
"            self._errno = box.errno()\n"
"            return false\n"
"        end\n"
"        return true\n"
"    end\n"
"\n"
"    if info.type == 2 then\n"
"        local res = ffi.C.setsockopt(self.fh,\n"
"            level, info.iname, value, ffi.sizeof('size_t'))\n"
"        if res < 0 then\n"
"            self._errno = box.errno()\n"
"            return false\n"
"        end\n"
"        return true\n"
"    end\n"
"\n"
"    if name == 'SO_LINGER' then\n"
"        error(\"Use s:linger(active[, timeout])\")\n"
"    end\n"
"    error(sprintf(\"Unsupported socket option: %s\", name))\n"
"end\n"
"\n"
"socket_methods.getsockopt = function(self, level, name)\n"
"    local info = get_ivalue(box.socket.internal.SO_OPT, name)\n"
"\n"
"    if info == nil then\n"
"        error(sprintf(\"Unknown socket option name: %s\", tostring(name)))\n"
"    end\n"
"\n"
"    self._errno = nil\n"
"\n"
"    if type(level) == 'string' then\n"
"        if level == 'SOL_SOCKET' then\n"
"            level = box.socket.internal.SOL_SOCKET\n"
"        else\n"
"            local p = ffi.C.getprotobyname(level)\n"
"            if p == nil then\n"
"                self._errno = box.errno(box.errno.EINVAL)\n"
"                return nil\n"
"            end\n"
"            level = p.p_proto\n"
"        end\n"
"    else\n"
"        level = tonumber(level)\n"
"    end\n"
"\n"
"\n"
"    if info.type == 1 then\n"
"        local value = ffi.new(\"int[1]\", 0)\n"
"        local len = ffi.new(\"size_t[1]\", ffi.sizeof('int'))\n"
"        local res = ffi.C.getsockopt(self.fh, level, info.iname, value, len)\n"
"\n"
"        if res < 0 then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"\n"
"        if len[0] ~= 4 then\n"
"            error(sprintf(\"Internal error: unexpected optlen: %d\", len[0]))\n"
"        end\n"
"        return value[0]\n"
"    end\n"
"\n"
"    if info.type == 2 then\n"
"        local value = ffi.new(\"char[256]\", { 0 })\n"
"        local len = ffi.new(\"size_t[1]\", 256)\n"
"        local res = ffi.C.getsockopt(self.fh, level, info.iname, value, len)\n"
"        if res < 0 then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"        return ffi.string(value, len[0])\n"
"    end\n"
"\n"
"    if name == 'SO_LINGER' then\n"
"        error(\"Use s:linger()\")\n"
"    end\n"
"    error(sprintf(\"Unsupported socket option: %s\", name))\n"
"end\n"
"\n"
"socket_methods.linger = function(self, active, timeout)\n"
"\n"
"    local info = box.socket.internal.SO_OPT.SO_LINGER\n"
"    self._errno = nil\n"
"    if active == nil then\n"
"        local value = ffi.new(\"linger_t[1]\")\n"
"        local len = ffi.new(\"size_t[1]\", 2 * ffi.sizeof('int'))\n"
"        local res = ffi.C.getsockopt(self.fh,\n"
"            box.socket.internal.SOL_SOCKET, info.iname, value, len)\n"
"        if res < 0 then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"        if value[0].active ~= 0 then\n"
"            active = true\n"
"        else\n"
"            active = false\n"
"        end\n"
"        return active, value[0].timeout\n"
"    end\n"
"\n"
"    if timeout == nil then\n"
"        timeout = 0\n"
"    end\n"
"\n"
"    local iactive\n"
"    if active then\n"
"        iactive = 1\n"
"    else\n"
"        iactive = 0\n"
"    end\n"
"\n"
"    local value = ffi.new(\"linger_t[1]\",\n"
"        { { active = iactive, timeout = timeout } })\n"
"    local len = 2 * ffi.sizeof('int')\n"
"    local res = ffi.C.setsockopt(self.fh,\n"
"        box.socket.internal.SOL_SOCKET, info.iname, value, len)\n"
"    if res < 0 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"\n"
"    return active, timeout\n"
"end\n"
"\n"
"socket_methods.accept = function(self)\n"
"\n"
"    self._errno = nil\n"
"\n"
"    local fh = ffi.C.accept(self.fh, nil, nil)\n"
"\n"
"    if fh < 1 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"\n"
"    -- Make socket to be non-blocked by default\n"
"    -- ignore result\n"
"    ffi.C.bsdsocket_nonblock(fh, 1)\n"
"\n"
"    local socket = { fh = fh }\n"
"    setmetatable(socket, box.socket.internal.socket_mt)\n"
"    return socket\n"
"end\n"
"\n"
"socket_methods.read = function(self, size, timeout)\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    if self.rbuf == nil then\n"
"        self.rbuf = ''\n"
"    end\n"
"\n"
"    if size == nil then\n"
"        size = 4294967295\n"
"    end\n"
"\n"
"    if string.len(self.rbuf) >= size then\n"
"        self._errno = nil\n"
"        local data = string.sub(self.rbuf, 1, size)\n"
"        self.rbuf = string.sub(self.rbuf, size + 1)\n"
"        return data\n"
"    end\n"
"\n"
"    while timeout > 0 do\n"
"        local started = box.time()\n"
"\n"
"        if not self:readable(timeout) then\n"
"            return nil\n"
"        end\n"
"\n"
"        timeout = timeout - ( box.time() - started )\n"
"\n"
"        local data = self:sysread(4096)\n"
"        if data ~= nil then\n"
"            self.rbuf = self.rbuf .. data\n"
"            if string.len(self.rbuf) >= size then\n"
"               data = string.sub(self.rbuf, 1, size)\n"
"               self.rbuf = string.sub(self.rbuf, size + 1)\n"
"               return data\n"
"            end\n"
"\n"
"            if string.len(data) == 0 then   -- eof\n"
"                data = self.rbuf\n"
"                self.rbuf = nil\n"
"                return data\n"
"            end\n"
"        elseif self:errno() ~= box.errno.EAGAIN then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"    end\n"
"    self._errno = box.errno.ETIMEDOUT\n"
"    return nil\n"
"end\n"
"\n"
"local function readline_check(self, eols, limit)\n"
"    local rbuf = self.rbuf\n"
"    if rbuf == nil then\n"
"        return nil\n"
"    end\n"
"    if string.len(rbuf) == 0 then\n"
"        return nil\n"
"    end\n"
"    for i, eol in pairs(eols) do\n"
"        if string.len(rbuf) >= string.len(eol) then\n"
"            local data = string.match(rbuf, \"^(.-\" .. eol .. \")\")\n"
"            if data ~= nil then\n"
"                if string.len(data) > limit then\n"
"                    return string.sub(data, 1, limit)\n"
"                end\n"
"                return data\n"
"            end\n"
"        end\n"
"    end\n"
"    return nil\n"
"end\n"
"\n"
"socket_methods.readline = function(self, limit, eol, timeout)\n"
"    if type(limit) == 'table' then -- :readline({eol}[, timeout ])\n"
"        if eol ~= nil then\n"
"            timeout = eol\n"
"            eol = limit\n"
"            limit = nil\n"
"        else\n"
"            eol = limit\n"
"            limit = nil\n"
"        end\n"
"    elseif eol ~= nil then\n"
"        if type(eol) ~= 'table' then\n"
"            eol = { eol }\n"
"        end\n"
"    end\n"
"\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    if self.rbuf == nil then\n"
"        self.rbuf = ''\n"
"    end\n"
"\n"
"    if limit == nil then\n"
"        limit = 4294967295\n"
"    end\n"
"\n"
"    self._errno = nil\n"
"    local data = readline_check(self, eol, limit)\n"
"    if data ~= nil then\n"
"        self.rbuf = string.sub(self.rbuf, string.len(data) + 1)\n"
"        return data\n"
"    end\n"
"\n"
"    local started = box.time()\n"
"    while timeout > 0 do\n"
"        local started = box.time()\n"
"\n"
"        if not self:readable(timeout) then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"\n"
"        timeout = timeout - ( box.time() - started )\n"
"\n"
"        local data = self:sysread(4096)\n"
"        if data ~= nil then\n"
"            self.rbuf = self.rbuf .. data\n"
"\n"
"            -- eof\n"
"            if string.len(data) == 0 then\n"
"                data = self.rbuf\n"
"                self.rbuf = nil\n"
"                return data\n"
"            end\n"
"\n"
"            data = readline_check(self, eol, limit)\n"
"            if data ~= nil then\n"
"                self.rbuf = string.sub(self.rbuf, string.len(data) + 1)\n"
"                return data\n"
"            end\n"
"\n"
"            -- limit\n"
"            if string.len(self.rbuf) >= limit then\n"
"               data = string.sub(self.rbuf, 1, limit)\n"
"               self.rbuf = string.sub(self.rbuf, limit + 1)\n"
"               return data\n"
"            end\n"
"\n"
"        elseif self:errno() ~= box.errno.EAGAIN then\n"
"            self._errno = box.errno()\n"
"            return nil\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"socket_methods.write = function(self, octets, timeout)\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"\n"
"    local started = box.time()\n"
"    while timeout > 0 and self:writable(timeout) do\n"
"        timeout = timeout - ( box.time() - started )\n"
"\n"
"        local written = self:syswrite(octets)\n"
"        if written == nil then\n"
"            if self:errno() ~= box.errno.EAGAIN then\n"
"                return false\n"
"            end\n"
"        end\n"
"\n"
"        if written == string.len(octets) then\n"
"            return true\n"
"        end\n"
"        if written > 0 then\n"
"            octets = string.sub(octets, written + 1)\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"socket_methods.send = function(self, octets, flags)\n"
"    local iflags = get_iflags(box.socket.internal.SEND_FLAGS, flags)\n"
"\n"
"    self._errno = nil\n"
"    local res = ffi.C.send(self.fh, octets, string.len(octets), iflags)\n"
"    if res == -1 then\n"
"        self._errno = box.errno()\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"socket_methods.recv = function(self, size, flags)\n"
"    local iflags = get_iflags(box.socket.internal.SEND_FLAGS, flags)\n"
"    if iflags == nil then\n"
"        self._errno = box.errno.EINVAL\n"
"        return nil\n"
"    end\n"
"\n"
"    if size == nil then\n"
"        size = 512\n"
"    end\n"
"    self._errno = nil\n"
"    local buf = ffi.new(\"char[?]\", size)\n"
"\n"
"    local res = ffi.C.recv(self.fh, buf, size, iflags)\n"
"\n"
"    if res == -1 then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"    return ffi.string(buf, res)\n"
"end\n"
"\n"
"socket_methods.recvfrom = function(self, size, flags)\n"
"    local iflags = get_iflags(box.socket.internal.SEND_FLAGS, flags)\n"
"    if iflags == nil then\n"
"        self._errno = box.errno.EINVAL\n"
"        return nil\n"
"    end\n"
"    self._errno = nil\n"
"    local res, from = box.socket.internal.recvfrom(self.fh, size, iflags)\n"
"    if res == nil then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"    return res, from\n"
"end\n"
"\n"
"socket_methods.sendto = function(self, host, port, octets, flags)\n"
"    local iflags = get_iflags(box.socket.internal.SEND_FLAGS, flags)\n"
"\n"
"    if iflags == nil then\n"
"        self._errno = box.errno.EINVAL\n"
"        return nil\n"
"    end\n"
"\n"
"    self._errno = nil\n"
"    if octets == nil or octets == '' then\n"
"        return true\n"
"    end\n"
"\n"
"    host = tostring(host)\n"
"    port = tostring(port)\n"
"    octets = tostring(octets)\n"
"\n"
"    addr_len[0] = ffi.sizeof(addr)\n"
"    local res = ffi.C.bsdsocket_local_resolve(host, port, addr, addr_len)\n"
"    if res == 0 then\n"
"        res = ffi.C.sendto(self.fh, octets, string.len(octets), iflags,\n"
"            addr, addr_len[0])\n"
"    end\n"
"    if res < 0 then\n"
"        self._errno = box.errno()\n"
"        return false\n"
"    end\n"
"    return true\n"
"end\n"
"\n"
"local function create_socket(self, domain, stype, proto)\n"
"    local idomain = get_ivalue(box.socket.internal.DOMAIN, domain)\n"
"    if idomain == nil then\n"
"        box.errno(box.errno.EINVAL)\n"
"        return nil\n"
"    end\n"
"\n"
"    local itype = get_ivalue(box.socket.internal.SO_TYPE, stype)\n"
"    if itype == nil then\n"
"        box.errno(box.errno.EINVAL)\n"
"        return nil\n"
"    end\n"
"\n"
"    local p = ffi.C.getprotobyname(proto)\n"
"    if p == nil then\n"
"        box.errno(box.errno.EINVAL)\n"
"        return nil\n"
"    end\n"
"    local iproto = p.p_proto\n"
"\n"
"    local fh = ffi.C.socket(idomain, itype, iproto)\n"
"    if fh < 0 then\n"
"        return nil\n"
"    end\n"
"\n"
"    -- Make socket to be non-blocked by default\n"
"    if ffi.C.bsdsocket_nonblock(fh, 1) < 0 then\n"
"        local errno = box.errno()\n"
"        ffi.C.close(fh)\n"
"        box.errno(errno)\n"
"        return nil\n"
"    end\n"
"\n"
"    local socket = { fh = fh }\n"
"    setmetatable(socket, box.socket.internal.socket_mt)\n"
"    return socket\n"
"end\n"
"\n"
"local function getaddrinfo(host, port, timeout, opts)\n"
"    local self = box.socket\n"
"\n"
"    if type(timeout) == 'table' and opts == nil then\n"
"        opts = timeout\n"
"        timeout = TIMEOUT_INFINITY\n"
"    elseif timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"    if port == nil then\n"
"        port = 0\n"
"    end\n"
"    local ga_opts = {}\n"
"    if opts ~= nil then\n"
"        if opts.type ~= nil then\n"
"            local itype = get_ivalue(box.socket.internal.SO_TYPE, opts.type)\n"
"            if itype == nil then\n"
"                self._errno = box.errno.EINVAL\n"
"                return nil\n"
"            end\n"
"            ga_opts.type = itype\n"
"        end\n"
"\n"
"        if opts.family ~= nil then\n"
"            local ifamily = get_ivalue(box.socket.internal.DOMAIN, opts.family)\n"
"            if ifamily == nil then\n"
"                self._errno = box.errno.EINVAL\n"
"                return nil\n"
"            end\n"
"            ga_opts.family = ifamily\n"
"        end\n"
"\n"
"        if opts.protocol ~= nil then\n"
"            local p = ffi.C.getprotobyname(opts.protocol)\n"
"            if p == nil then\n"
"                self._errno = box.errno(box.errno.EINVAL)\n"
"                return nil\n"
"            end\n"
"            ga_opts.protocol = p.p_proto\n"
"        end\n"
"\n"
"        if opts.flags ~= nil then\n"
"            ga_opts.flags =\n"
"                get_iflags(box.socket.internal.AI_FLAGS, opts.flags)\n"
"            if ga_opts.flags == nil then\n"
"                self._errno = box.errno()\n"
"                return nil\n"
"            end\n"
"        end\n"
"\n"
"    end\n"
"\n"
"    local r = box.socket.internal.getaddrinfo(host, port, timeout, ga_opts)\n"
"    if r == nil then\n"
"        self._errno = box.errno()\n"
"    else\n"
"        self._errno = nil\n"
"    end\n"
"    return r\n"
"end\n"
"\n"
"socket_methods.name = function(self)\n"
"    local aka = box.socket.internal.name(self.fh)\n"
"    if aka == nil then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"    self._errno = nil\n"
"    return aka\n"
"end\n"
"\n"
"socket_methods.peer = function(self)\n"
"    local peer = box.socket.internal.peer(self.fh)\n"
"    if peer == nil then\n"
"        self._errno = box.errno()\n"
"        return nil\n"
"    end\n"
"    self._errno = nil\n"
"    return peer\n"
"end\n"
"\n"
"-- tcp connector\n"
"local function tcp_connect(host, port, timeout)\n"
"    if host == 'unix/' and port ~= nil then\n"
"        local s = package.loaded.socket('AF_UNIX', 'SOCK_STREAM', 'ip')\n"
"        if s == nil then\n"
"            return nil\n"
"        end\n"
"        if s:sysconnect(host, port) then\n"
"            return s\n"
"        else\n"
"            local errno = box.errno()\n"
"            s:close()\n"
"            box.errno(errno)\n"
"            return nil\n"
"        end\n"
"    end\n"
"\n"
"    if timeout == nil then\n"
"        timeout = TIMEOUT_INFINITY\n"
"    end\n"
"    local started = box.time()\n"
"    local dns = box.socket.getaddrinfo(host, port, timeout,\n"
"                                    { type = 'SOCK_STREAM', protocol = 'tcp' })\n"
"    if dns == nil then\n"
"        return nil\n"
"    end\n"
"\n"
"\n"
"\n"
"    for i, remote in pairs(dns) do\n"
"\n"
"        timeout = timeout - (box.time() - started)\n"
"        if timeout <= 0 then\n"
"            box.errno(box.errno.ETIMEDOUT)\n"
"            return nil\n"
"        end\n"
"\n"
"        started = box.time()\n"
"\n"
"        local s = box.socket(remote.family, remote.type, 'tcp')\n"
"        if s == nil then\n"
"            return nil\n"
"        end\n"
"\n"
"        if s:sysconnect(remote.host, remote.port) then\n"
"\n"
"            timeout = timeout - (box.time() - started)\n"
"            if timeout <= 0 then\n"
"                s:close()\n"
"                break\n"
"            end\n"
"\n"
"            if s:writable(timeout) then\n"
"                return s\n"
"            end\n"
"        end\n"
"\n"
"        local save_errno = box.errno()\n"
"        s:close()\n"
"        box.errno(save_errno)\n"
"    end\n"
"\n"
"    if timeout <= 0 then\n"
"        box.errno(box.errno.ETIMEDOUT)\n"
"    end\n"
"    return nil\n"
"end\n"
"\n"
"box.socket.internal = {\n"
"    socket_mt   = {\n"
"        __index     = socket_methods,\n"
"        __tostring  = function(self)\n"
"            local save_errno = self._errno\n"
"            local name = sprintf(\"fd %d\", self.fh)\n"
"            local aka = self:name()\n"
"            if aka ~= nil then\n"
"                name = sprintf(\"%s, aka %s:%s\", name, aka.host, aka.port)\n"
"            end\n"
"            local peer = self:peer(self)\n"
"            if peer ~= nil then\n"
"                name = sprintf(\"%s, peer %s:%s\", name, peer.host, peer.port)\n"
"            end\n"
"            self._errno = save_errno\n"
"            return name\n"
"        end\n"
"    },\n"
"}\n"
"\n"
"setmetatable(box.socket, {\n"
"    __call = create_socket,\n"
"    __index = {\n"
"        getaddrinfo = getaddrinfo,\n"
"        tcp_connect = tcp_connect\n"
"    }\n"
"})\n"
"\n"
"end\n"
""
;
